using System;
using System.Threading;
using System.Collections;

namespace RemotingExample.Security
{
	/// <summary>
	/// SecurityContextContainer contains ISecurityContext objects (users sessions). On client and server side.
	/// 
	/// </summary>
	public class SecurityContextContainer : IDisposable,ICollection, IEnumerable
	{
		
		Hashtable contextsTable;
		private const int sessionTimeOut = 30; //After timeout expired session will be deleted
		private const int MillisecondsBetweenOldAgeChecks = 1000; //Interval between session cleanup thread starts 
		private Thread _cleanupThread = null; //Session cleanup thread
		static SecurityContextContainer instance; 

		public int SessionTimeOut
		{
			
			get 
			{
				return sessionTimeOut;
			}
		}
		
		
		public static SecurityContextContainer GetInstance()
		{
			
			if (instance==null)
			{
				instance= new SecurityContextContainer();
			}
			return instance;
		}
		
		
		SecurityContextContainer()
		{
			contextsTable = new Hashtable();
			// create a new cleanup thread
			_cleanupThread = new Thread(new ThreadStart(this.CleanupThread));
			// Start the thread
			_cleanupThread.Start();

		}


		~SecurityContextContainer()
		{
			Dispose(false);
			
		}

		// IDisposable
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);

		}
		
		
		private  void Dispose(bool disposing)
		{
			if (disposing == true)
			{
				if (_cleanupThread != null)
				{
					_cleanupThread.Abort();
					_cleanupThread.Join();
					_cleanupThread = null;
				}
			
				foreach (DictionaryEntry entry in this)
				{
					ISecurityContext context = (ISecurityContext)entry.Value;
					context.Dispose();
					context = null;
				}
				GC.Collect();
			}
		
		}

		//private methods
		ISecurityContext GetOldEntry()
		{
			DateTime currentTime = DateTime.Now;
			ISecurityContext context;
			TimeSpan lifeTime;
			foreach (DictionaryEntry entry in this)
			{
				context = (ISecurityContext)entry.Value;
				lifeTime = currentTime - context.LastActivity;
				if (lifeTime.Minutes >= sessionTimeOut)
					return context;
			}

			return null;
		}

		//Cleanup old sessions
		private void CleanupThread()
		{
			while (true)
			{
				Thread.Sleep(MillisecondsBetweenOldAgeChecks);

				ISecurityContext context;
				do 
				{
					context = GetOldEntry();
					if (context != null)
					{
						this.Remove(context.SessionID);
						context.Dispose();
					}
				} while (context != null);
			}
		}



		
	
		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return this.contextsTable.GetEnumerator ();
		}

		#endregion
	
		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return false;
			}
		}

		
		public ISecurityContext this [string key] 
		{
			get 
			{ 
				object context = contextsTable[key];
				if (context == null)
				{
					return null;
				}
				else
				{
					return (ISecurityContext)context; 
				}
			}
			set { contextsTable[key] = value; }
		}

		public ISecurityContext this [int index] 
		{
			get 
			{ 
				object context = contextsTable[index];
				if (context == null)
				{
					return null;
				}
				else
				{
					return (ISecurityContext)context; 
				}
			}
			set { contextsTable[index] = value; }
		}

		public int Count
		{
			get
			{
				return contextsTable.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			throw new System.NotImplementedException("CopyTo Not implemented");
				
		}	

		public object SyncRoot
		{
			get
			{
				return this;
			}
		}

		#endregion

		public void Remove (string name)
		{
			contextsTable.Remove (name);
		}

		public void RemoveAll ()
		{
			contextsTable.Clear();

		}

		public void Add(string contextID, ISecurityContext value)
		{
			this.contextsTable[contextID] = value;
		}

	}
}
